from django import forms
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.contrib.auth.models import User

from django_modal_actions.mixins import ModalActionMixin, modal_action


class CustomForm(forms.Form):
    name = forms.CharField(label="Name", required=True)

    def clean_name(self):
        name = self.cleaned_data["name"]
        if name == "bad":
            raise forms.ValidationError("Name cannot be 'bad'")
        return name


class UserAdmin(ModalActionMixin, BaseUserAdmin):
    modal_actions = ["object_action", "object_action_with_form_class"]
    list_modal_actions = ["list_action", "list_action_with_form_class"]

    @modal_action(modal_header="Object Action")
    def object_action(self, request, obj, form_data=None):
        return "Object action works"

    @modal_action(modal_header="List Action")
    def list_action(self, request, queryset, form_data=None):
        return "List action works"

    @modal_action(modal_header="Object Action with Form Class", form_class=CustomForm)
    def object_action_with_form_class(self, request, obj, form_data=None):
        return "Object action with form class works"

    @modal_action(modal_header="List Action with Form Class", form_class=CustomForm)
    def list_action_with_form_class(self, request, queryset, form_data=None):
        return f"List action with form class works on {queryset.count()} items"


admin.site.unregister(User)
admin.site.register(User, UserAdmin)



import json
from functools import wraps
from typing import Any, Callable, Dict, List, Optional, Type, Union

from django.contrib import messages
from django.http import HttpRequest, JsonResponse
from django.template.loader import render_to_string
from django.urls import path, reverse
from django.utils.html import format_html


class ModalActionMixin:
    modal_actions: List[str] = []
    list_modal_actions: List[str] = []
    change_form_template: str = "django_modal_actions/change_form.html"
    change_list_template: str = "django_modal_actions/change_list.html"

    def get_urls(self) -> List[path]:
        urls: List[path] = super().get_urls()
        custom_urls = [
            path(
                "<path:object_id>/modal-action/<str:action>/",
                self.admin_site.admin_view(self.get_modal_content),
                name=f"{self.model._meta.app_label}_{self.model._meta.model_name}_modal_action",
            ),
            path(
                "<path:object_id>/execute-action/<str:action>/",
                self.admin_site.admin_view(self.execute_modal_action),
                name=f"{self.model._meta.app_label}_{self.model._meta.model_name}_execute_action",
            ),
            path(
                "list-modal-action/<str:action>/",
                self.admin_site.admin_view(self.get_modal_content),
                name=f"{self.model._meta.app_label}_{self.model._meta.model_name}_list_modal_action",
            ),
            path(
                "execute-list-action/<str:action>/",
                self.admin_site.admin_view(self.execute_modal_action),
                name=f"{self.model._meta.app_label}_{self.model._meta.model_name}_execute_list_action",
            ),
        ]
        return custom_urls + urls

    def get_modal_content(
        self, request: HttpRequest, action: str, object_id: Optional[str] = None
    ) -> JsonResponse:
        obj = self.get_object(request, object_id) if object_id else None
        action_func: Callable = getattr(self, action)
        form_class: Optional[Type] = getattr(action_func, "form_class", None)
        form = form_class(request.POST or None) if form_class else None

        if obj is None:
            selected_ids = json.loads(request.GET.get("selected_ids", "[]"))
            queryset = self.model.objects.filter(pk__in=selected_ids)
            description = (
                getattr(action_func, "modal_description", None)
                or f"Are you sure you want to perform this action on {queryset.count()} items?"
            )
        else:
            description = (
                getattr(action_func, "modal_description", None)
                or f"Are you sure you want to perform this action on {obj}?"
            )

        context = {
            "object": obj,
            "action": action,
            "action_name": getattr(action_func, "modal_header", None)
            or action.replace("_", " ").title(),
            "description": description,
            "opts": self.model._meta,
            "form": form,
            "selected_ids": json.dumps(selected_ids) if obj is None else None,
        }
        content = render_to_string(
            "django_modal_actions/modal_actions.html", context, request
        )
        return JsonResponse({"content": content})

    def execute_modal_action(
        self, request: HttpRequest, action: str, object_id: Optional[str] = None
    ) -> JsonResponse:
        action_func: Callable = getattr(self, action)
        form_class: Optional[Type] = getattr(action_func, "form_class", None)
        try:
            if object_id:
                obj = self.get_object(request, object_id)
                queryset_or_obj = obj
            else:
                selected_ids = json.loads(request.POST.get("selected_ids", "[]"))
                queryset_or_obj = self.model.objects.filter(pk__in=selected_ids)

            if not self.has_action_permission(
                request, action, obj if object_id else None
            ):
                return JsonResponse(
                    {"success": False, "errors": {"__all__": ["Permission denied"]}}
                )
            if form_class:
                form = form_class(request.POST)
                if form.is_valid():
                    response = action_func(request, queryset_or_obj, form.cleaned_data)
                    self.message_user(request, str(response), messages.SUCCESS)
                    return JsonResponse({"success": True})
                return JsonResponse({"success": False, "errors": form.errors})
            response = action_func(request, queryset_or_obj)
            self.message_user(request, str(response), messages.SUCCESS)
            return JsonResponse({"success": True})
        except Exception as e:
            return JsonResponse({"success": False, "errors": {"__all__": [str(e)]}})

    def get_modal_action_buttons(self, obj=None) -> str:
        buttons: List[str] = []
        actions = self.list_modal_actions if obj is None else self.modal_actions
        for action in actions:
            if self.has_action_permission(self.request, action, obj):
                func = getattr(self, action)
                if obj:
                    url = reverse(
                        f"admin:{obj._meta.app_label}_{obj._meta.model_name}_modal_action",
                        args=[obj.pk, action],
                    )
                else:
                    url = reverse(
                        f"admin:{self.model._meta.app_label}_{self.model._meta.model_name}_list_modal_action",
                        args=[action],
                    )
                buttons.append(
                    format_html(
                        '<a class="historylink modal-action-button" href="{}">{}</a>',
                        url,
                        getattr(
                            func, "short_description", action.replace("_", " ").upper()
                        ),
                    )
                )
        return format_html("".join(buttons))

    def change_view(
        self,
        request: HttpRequest,
        object_id: str,
        form_url: str = "",
        extra_context: Optional[Dict[str, Any]] = None,
    ):
        self.request = request
        extra_context = extra_context or {}
        extra_context["modal_action_buttons"] = self.get_modal_action_buttons(
            self.get_object(request, object_id)
        )
        return super().change_view(
            request, object_id, form_url, extra_context=extra_context
        )

    def changelist_view(
        self, request: HttpRequest, extra_context: Optional[Dict[str, Any]] = None
    ):
        self.request = request
        extra_context = extra_context or {}
        extra_context["list_modal_action_buttons"] = self.get_modal_action_buttons()
        return super().changelist_view(request, extra_context=extra_context)

    def has_action_permission(
        self, request: HttpRequest, action: str, obj: Optional[object] = None
    ) -> bool:
        action_func: Optional[Callable] = getattr(self, action, None)
        if not action_func:
            return True

        permissions = getattr(action_func, "permissions", None)
        if not permissions:
            return True

        if not isinstance(permissions, (list, tuple)):
            permissions = [permissions]

        try:
            return all(p(request, obj) for p in permissions)
        except TypeError as e:
            raise ValueError(f"Invalid permission check: {e}") from e

    class Media:
        js = ("admin/js/jquery.init.js", "django_modal_actions/js/modal_actions.js")
        css = {"all": ("django_modal_actions/css/modal_actions.css",)}


def modal_action(
    modal_header: Optional[str] = None,
    modal_description: Optional[str] = None,
    permissions: Optional[Union[Callable, List[Callable]]] = None,
    form_class: Optional[Type] = None,
):
    def decorator(func):
        @wraps(func)
        def wrapper(self, request, queryset_or_obj, form_data=None):
            return func(self, request, queryset_or_obj, form_data)

        wrapper.modal_header = modal_header
        wrapper.modal_description = modal_description
        wrapper.permissions = permissions
        wrapper.form_class = form_class
        return wrapper

    return decorator



import os

from django.apps import AppConfig


class DjangoModalActionsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "django_modal_actions"
    path = os.path.dirname(os.path.abspath(__file__))



# ruff: noqa: E402
import os

import django

os.environ["DJANGO_SETTINGS_MODULE"] = "django_modal_actions.tests.test_settings"
django.setup()

import time

from django.contrib.auth.models import User
from django.contrib.staticfiles.testing import StaticLiveServerTestCase
from django.core.management import call_command
from django.db import connection
from django.test import Client
from django.urls import reverse
from selenium import webdriver
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait


class DjangoModalActionsTests(StaticLiveServerTestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        if connection.vendor == "sqlite":
            cursor = connection.cursor()
            cursor.execute("PRAGMA foreign_keys = OFF;")
            cursor.close()

        call_command("migrate")

        chrome_options = Options()
        chrome_options.add_argument("--headless")
        cls.selenium = webdriver.Chrome(options=chrome_options)
        cls.selenium.implicitly_wait(10)

    @classmethod
    def tearDownClass(cls):
        cls.selenium.quit()
        super().tearDownClass()

    def setUp(self):
        self.admin_user = User.objects.create_superuser(
            username="admin", email="admin@example.com", password="adminpassword"
        )
        self.client = Client()
        self.client.login(username="admin", password="adminpassword")

        session_id = self.client.cookies["sessionid"].value
        self.selenium.get(f"{self.live_server_url}/admin/")
        self.selenium.add_cookie(
            {"name": "sessionid", "value": session_id, "path": "/"}
        )

    def open_modal(self, action_name):
        modal_button = WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable((By.LINK_TEXT, action_name))
        )
        modal_button.click()
        WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.ID, "modal-action"))
        )
        time.sleep(1)  # Allow for any animations to complete

    def submit_form(self, name_value):
        name_field = WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.ID, "id_name"))
        )
        name_field.clear()
        name_field.send_keys(name_value)
        submit_button = WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable(
                (By.CSS_SELECTOR, "#modal-action button[type='submit']")
            )
        )
        self.selenium.execute_script("arguments[0].click();", submit_button)

    def test_list_modal_action_button_appears(self):
        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
        modal_button = self.selenium.find_element(
            By.LINK_TEXT, "LIST ACTION WITH FORM CLASS"
        )
        self.assertIsNotNone(modal_button)

    def test_object_modal_action_button_appears(self):
        user = User.objects.first()
        self.selenium.get(
            self.live_server_url + reverse("admin:auth_user_change", args=[user.id])
        )
        modal_button = self.selenium.find_element(By.LINK_TEXT, "OBJECT ACTION")
        self.assertIsNotNone(modal_button)

    def test_form_validation_invalid_input(self):
        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
        self.open_modal("LIST ACTION WITH FORM CLASS")
        self.submit_form("bad")

        error_list = WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.CLASS_NAME, "errorlist"))
        )
        self.assertIn("Name cannot be 'bad'", error_list.text)

    def test_form_submission_valid_input(self):
        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
        self.open_modal("LIST ACTION WITH FORM CLASS")
        self.submit_form("good_name")

        # Wait for the modal to close
        WebDriverWait(self.selenium, 10).until(
            EC.invisibility_of_element_located((By.ID, "modal-action"))
        )

        # Check for success message
        success_message = WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.CLASS_NAME, "success"))
        )
        self.assertIn(
            "List action with form class works on 0 items", success_message.text
        )

    def test_modal_closes_on_cancel(self):
        self.selenium.get(self.live_server_url + reverse("admin:auth_user_changelist"))
        self.open_modal("LIST ACTION WITH FORM CLASS")

        # Ensure the modal is visible
        modal = WebDriverWait(self.selenium, 10).until(
            EC.visibility_of_element_located((By.ID, "modal-action"))
        )
        self.assertTrue(modal.is_displayed())

        cancel_button = WebDriverWait(self.selenium, 10).until(
            EC.element_to_be_clickable((By.CSS_SELECTOR, "#modal-action button.cancel"))
        )
        cancel_button.click()

        # Wait for the modal to become invisible
        WebDriverWait(self.selenium, 10).until(
            EC.invisibility_of_element_located((By.ID, "modal-action"))
        )

        # Check if the modal is no longer visible
        modal = self.selenium.find_element(By.ID, "modal-action")
        self.assertFalse(modal.is_displayed())

        # Check if the modal-backdrop is removed
        try:
            self.selenium.find_element(By.CLASS_NAME, "modal-backdrop")
            self.fail("Modal backdrop is still present")
        except NoSuchElementException:
            pass  # This is expected

    def test_object_modal_form_submission(self):
        user = User.objects.first()
        self.selenium.get(
            self.live_server_url + reverse("admin:auth_user_change", args=[user.id])
        )
        self.open_modal("OBJECT ACTION WITH FORM CLASS")
        self.submit_form("good_name")

        # Wait for the modal to close
        WebDriverWait(self.selenium, 10).until(
            EC.invisibility_of_element_located((By.ID, "modal-action"))
        )

        # Check for success message
        success_message = WebDriverWait(self.selenium, 10).until(
            EC.presence_of_element_located((By.CLASS_NAME, "success"))
        )
        self.assertIn("Object action with form class works", success_message.text)


if __name__ == "__main__":
    import unittest

    unittest.main()
